import json
import requests as req

class Util():
    url = "https://api.ds2.ai"

    def predict(self, request_data, predict_url, data, return_type="info"):

        if type(data) == dict:
            request_data["parameter"] = data
            response = {}
            response_raw = json.loads(self._predict(predict_url['predict'], request_data).json())
            if response_raw.get('이상값칼럼'):
                response['outliar'] = response_raw['이상값칼럼']
            for key, item in response_raw.items():
                if key.endswith('__예측값'):
                    response[key.replace('__예측값', '__predicted')] = item
            return response
        else:
            if data.startswith(('http://', 'https://')):
                request_data['url'] = data
                if return_type == "info":
                    inference_results_raw = self._predict(predict_url['predictimagebyurlinfo'], request_data).json()
                    inference_results = []
                    boxes = inference_results_raw.get("boxes", [])
                    masks = inference_results_raw.get("masks", [])
                    for index, inference_result_raw in enumerate(inference_results_raw.get('prediction', [])):
                        inference_result = {
                            "class_name": inference_result_raw,
                            "score": inference_result_raw,
                        }
                        if boxes:
                            inference_result['box'] = boxes[index]
                        if masks:
                            inference_result['mask'] = masks[index]
                        inference_results.append(Instance(inference_result))

                    return inference_results
                elif return_type == "xai":
                    return self._predict(predict_url['predictimagebyurlxai'], request_data).content
                else:
                    return self._predict(predict_url['predictimagebyurl'], request_data).content
            else:
                with open(data, "rb") as f:
                    file_content = f.read()
                    request_data["filename"] = data.split('/')[-1] if '/' in data else data
                    if data.endswith('.csv'):
                        return self._predict(predict_url['predictall'], request_data,
                                             file_content=file_content)
                    if data.endswith((".jpg", ".jpeg", ".png", "gif", ".mp4", ".mov")):
                        if return_type == "info":
                            inference_results_raw = self._predict(predict_url['predictimageinfo'], request_data,
                                             file_content=file_content).json()
                            inference_results = []
                            boxes = inference_results_raw.get("boxes", [])
                            polygons = inference_results_raw.get("polygons", [])
                            for index, inference_result_raw in enumerate(inference_results_raw.get('prediction', [])):
                                inference_result = {
                                    "class_name": inference_result_raw,
                                    "score": inference_results_raw['scores'][index],
                                }
                                if boxes:
                                    inference_result['box'] = boxes[index]
                                if polygons:
                                    inference_result['polygon'] = polygons[index]
                                inference_results.append(Instance(inference_result))

                            return inference_results

                        elif return_type == "xai":
                            return self._predict(predict_url['predictimagexai'], request_data,
                                             file_content=file_content).content
                        else:
                            return self._predict(predict_url['predictimage'], request_data,
                                             file_content=file_content).content

    def _predict(self, target_url, request_data, file_content=None):
        if file_content:
            return req.post(target_url, files={'file': file_content}, data=request_data, stream=True)
        else:
            return req.post(target_url, data=json.dumps(request_data))

class Asynctask(object):
    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token

    def __repr__(self):
        return str(self.id)

    def refresh(self):
        return Asynctask(req.get(f"{self.url}/asynctasks/{self.id}/",
                                 params={"token": self.user_token}).json(), self.user, url=self.url)


class MarketModel(object):
    utilClass = Util()

    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.name = info['name_en']
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token

    def __repr__(self):
        return str(self.name)

    def predict(self, data, return_type="info"):
        request_data = {
            "apptoken": self.user.appTokenCode,
            "modelid": self.id,
            "userId": self.user.id,
        }

        predict_url = {
            "predict": f"{self.url}/market/predict/",
            "predictimagebyurl": f"{self.url}/market/predictimagebyurl/",
            "predictimagebyurlxai": f"{self.url}/market/predictimagebyurlxai/",
            "predictimagebyurlinfo": f"{self.url}/market/predictimagebyurlinfo/",
            "predictall": f"{self.url}/market/predictall/",
            "predictimage": f"{self.url}/market/predictimage/",
            "predictimagexai": f"{self.url}/market/predictimagexai/",
            "predictimageinfo": f"{self.url}/market/predictimageinfo/",
        }

        return self.utilClass.predict(request_data, predict_url, data, return_type=return_type)

class User(object):
    def __init__(self, info):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.token = info['token']
        self.__dict__.update(info)

class Instance(object):

    def __init__(self, info):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.class_name = info['class_name']

    def __repr__(self):
        return str(self.class_name)



# Copyright 2020 The DS2AI Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__version__ = "1.0.0"

import time
from .util import Util, Asynctask, User, MarketModel
from .LabelingAI import Labelproject, Labelfile
from .ClickAI import Project, Model, Jupyterproject
from .SkyhubAI import Opsproject
from .DS2dataset import Dataconnector
import requests as req
import json

class DS2():

    def __init__(self, apptoken, url=None):
        self.utilClass = Util()
        self.url = url if url else self.utilClass.url
        self.apptoken = apptoken
        self.user = self.get_user_info()
        self.user_token = self.user.token

    def get_user_info(self):
        return User(req.get(f"{self.url}/auth/", params={"apptoken": self.apptoken}).json())

    def create_project(self, data_file, predict_column_name=None, frame=None, training_method=None):

        dataconnector = self.create_dataconnector(data_file, predict_column_name=predict_column_name, frame=frame)
        return self.create_project_by_dataconnector(dataconnector, training_method)

    def create_project_by_dataconnector(self, dataconnector, training_method):

        if isinstance(dataconnector, int):
            dataconnector_id = dataconnector
        else:
            dataconnector_id = dataconnector.id

        return Project(req.post(f"{self.url}/projectfromdataconnectors/",
                                    params={"token": self.user_token},
                                    data=json.dumps({
                                          'dataconnectors': [dataconnector_id],
                                          'trainingMethod': training_method
                                          })).json(), self.user, url=self.url)

    def load_model(self, model_file):

        with open(model_file, "rb") as f:
            file_content = f.read()
            return Project(req.post(f"{self.url}/projectswithmodelfile/",
                                    params={"token": self.user_token}, files={'file': file_content},
                                    data={'filename': model_file.split("/")[-1] if "/" in model_file else model_file},
                                    stream=True).json(), self.user, url=self.url)

    def get_projects(self, count=25, start=1, desc=True):
        items = []
        items_raw = req.get(f"{self.url}/projects/",
                params={"token": self.user_token, "start": start, "page": count, "desc": desc}).json()['projects']
        for item_raw in items_raw:
            items.append(Project(item_raw, self.user, url=self.url))
        return items

    def get_project(self, project_id):
        return Project(req.get(f"{self.url}/projects/{project_id}/", params={"token": self.user_token}).json(), self.user, url=self.url)

    def get_model(self, model_id):
        return Model(req.get(f"{self.url}/models/{model_id}/", params={"token": self.user_token}).json(), self.user, url=self.url)

    def get_quick_models(self, count=25, start=1, desc=True):
        items = []
        items_raw = req.get(f"{self.url}/market-models/",
                    params={"token": self.user_token, "start": start, "page": count, "desc": desc, "is_quick_model":True}
                    ).json()['market_models']
        for item_raw in items_raw:
            items.append(MarketModel(item_raw, self.user))
        return items

    def get_quick_model_by_slug_name(self, slug_name):
        return MarketModel(req.get(f"{self.url}/marketmodels/slug/{slug_name}/",
                                   params={"token": self.user_token}).json(), self.user, url=self.url)

    def create_dataconnector(self, data_file, has_label_data=False, predict_column_name=None, frame=60):
        with open(data_file, "rb") as f:
            file_content = f.read()
            return Dataconnector(req.post(f"{self.url}/dataconnectorswithfile/",
                                    files={'file': file_content},
                                    data={'token': self.user_token,
                                          'filename': data_file.split("/")[-1] if "/" in data_file else data_file,
                                          'dataconnectorName': data_file.split("/")[-1] if "/" in data_file else data_file,
                                          'hasLabelData': has_label_data,
                                          'predictColumnName': predict_column_name,
                                          'frameValue': frame,
                                          }, stream=True).json(), self.user, url=self.url)

    def get_dataconnectors(self, count=25, start=1, desc=True):
        items = []
        items_raw = req.get(f"{self.url}/dataconnectors/",
                params={"token": self.user_token, "start": start, "page": count, "desc": desc}).json()['dataconnectors']
        for item_raw in items_raw:
            items.append(Dataconnector(item_raw, self.user, url=self.url))
        return items

    def get_dataconnector(self, dataconnector_id):
        return Dataconnector(req.get(f"{self.url}/dataconnectors/{dataconnector_id}/",
                                     params={"token": self.user_token}).json(), self.user, url=self.url)

    def create_labelproject(self, data_file=None, dataconnector=None, dataconnectors=None,
                            training_method=None, name=None, frame=60):

        if dataconnector:
            if isinstance(dataconnector, int):
                dataconnector = self.get_dataconnector(dataconnector)
            dataconnectors = [dataconnector.id]

            if not training_method:
                if dataconnector.dataconnectorName.endswith('.csv'):
                    training_method = "normal_regression"
                else:
                    training_method = "object_detection"

        if dataconnectors:

            if not name:
                name = f"Label Project from dataconnectors : {str(dataconnectors)}"

            return Labelproject(req.post(f"{self.url}/labelproject-from-dataconnectors/",
                                    params={"token": self.user_token},
                                    data=json.dumps({'dataconnectors': dataconnectors,
                                          'workapp': training_method,
                                          'name': name,
                                          'frame_value': frame,
                                          })).json(), self.user, url=self.url)

        elif data_file:
            if not name:
                name = f"Label Project from file : {str(data_file.split('/')[-1] if '/' in data_file else data_file)}"

            file_content = open(data_file, "r")
            return Labelproject(req.post(f"{self.url}/labelprojects/",
                                    params={"token": self.user_token},
                                    files={'files': file_content},
                                    data={
                                          'workapp': training_method,
                                          'frame_value': frame,
                                          'name': name,
                                          }, stream=True).json()['labelproject'], self.user, url=self.url)


        else:
            raise("You need to choose dataconnectors or files.")

    def get_labelprojects(self, count=25, start=1, desc=True):
        items = []
        items_raw = req.get(f"{self.url}/labelprojects/",
                params={"token": self.user_token, "start": start, "page": count, "desc": desc}).json()['projects']
        for item_raw in items_raw:
            items.append(Labelproject(item_raw, self.user, url=self.url))
        return items

    def get_labelproject(self, labelproject_id):
        return Labelproject(req.get(f"{self.url}/labelprojects/{labelproject_id}/",
                                     params={"token": self.user_token}).json(), self.user, url=self.url)

    def get_opsprojects(self, count=25, start=1, desc=True):
        items = []
        items_raw = req.get(f"{self.url}/opsprojects/",
                params={"token": self.user_token, "start": start, "page": count, "desc": desc}).json()['projects']
        for item_raw in items_raw:
            items.append(Opsproject(item_raw, self.user, url=self.url))
        return items

    def get_opsproject(self, opsproject_id):
        return Opsproject(req.get(f"{self.url}/opsprojects/{opsproject_id}/",
                                     params={"token": self.user_token}).json(), self.user, url=self.url)

    def get_jupyterprojects(self, count=25, start=1, desc=True):
        items = []
        items_raw = req.get(f"{self.url}/jupyterprojects/",
                params={"token": self.user_token, "start": start, "page": count, "desc": desc}).json()['projects']
        for item_raw in items_raw:
            items.append(Jupyterproject(item_raw, self.user, url=self.url))
        return items

    def get_jupyterproject(self, jupyterproject_id):
        return Jupyterproject(req.get(f"{self.url}/jupyterprojects/{jupyterproject_id}/",
                                     params={"token": self.user_token}).json(), self.user, url=self.url)

    def get_asynctasks(self, count=25, start=1, desc=True, tasktype="all"):
        items = []
        items_raw = req.get(f"{self.url}/asynctaskall/",
                params={"token": self.user_token, "start": start, "page": count, "desc": desc,
                        "tasktype": tasktype}).json()
        for item_raw in items_raw['asynctasks']:
            items.append(Asynctask(item_raw, self.user, url=self.url))
        return items

    def get_asynctask(self, asynctask_id):
        return Asynctask(req.get(f"{self.url}/asynctasks/{asynctask_id}/",
                                     params={"token": self.user_token}).json(), self.user, url=self.url)

    def start_auto_labeling(self, data_file, amount, has_label_data=False, predict_column_name=None, frame=60,
                          ai_type="general", autolabeling_type="box", general_ai_type="person",
                          model_id=None, custom_ai_stage=0, preprocessing_ai_type={}, labeling_class=[],
                          training_method="object_detection", name='', description=''
                          ):

        dataconnector = self.create_dataconnector(data_file, has_label_data=has_label_data, predict_column_name=predict_column_name)
        print("The data is being processed now. It will take a while. (Mostly less than 5 minutes.)")
        is_uploaded = False
        for i in range(100):
            time.sleep(5)
            dataconnector = self.get_dataconnector(dataconnector.id)
            if dataconnector.status == 100:
                is_uploaded = True
                break

        if not is_uploaded:
            raise ("The training data is being processed now. Please retry with ds2.train() when the data is ready. When it is ready, dataconnector.status will return 100.")

        if not name:
            name = f"label project from dataconnector {dataconnector.id}"

        return Asynctask(req.post(f"{self.url}/start-auto-labeling/",
                                params={"token": self.user_token},
                                data=json.dumps({
                                      'dataconnectors': [dataconnector.id],
                                      'predictColumnName': predict_column_name,
                                      'frameValue': frame,
                                      'workapp': training_method,
                                      'autolabeling_amount': amount,
                                      'autolabeling_ai_type': ai_type,
                                      'autolabeling_type': autolabeling_type,
                                      'general_ai_type': general_ai_type,
                                      'model_id': model_id,
                                      'custom_ai_stage': custom_ai_stage,
                                      'preprocessing_ai_type': preprocessing_ai_type,
                                      'labeling_class': labeling_class,
                                      'name': name,
                                      'description': description,
                                      })).json(), self.user, url=self.url)


    def train(self, data_file, training_method, value_for_predict, option="accuracy", frame=60):

        dataconnector = self.create_dataconnector(data_file, has_label_data=True, predict_column_name=value_for_predict)
        print("The data is being processed now. It will take a while. (Mostly less than 5 minutes.)")
        is_uploaded = False
        for i in range(100):
            time.sleep(5)
            dataconnector = self.get_dataconnector(dataconnector.id)
            if dataconnector.status == 100:
                is_uploaded = True
                break

        if not is_uploaded:
            raise ("The training data is being processed now. Please retry with ds2.train() when the data is ready. When it is ready, dataconnector.status will return 100.")

        return Project(req.post(f"{self.url}/train-from-data/",
                            params={"token": self.user_token},
                            data=json.dumps({
                                  'trainingMethod': training_method,
                                  'valueForPredict': value_for_predict,
                                  'dataconnector': dataconnector.id,
                                  'option': option,
                                  'frameValue': frame,
                                  })).json(), self.user, url=self.url)

    def deploy(self, model_file, name=None, cloud_type="AWS", region="us-west-1", server_type="g4dn.xlarge"):

        if "AWS" != cloud_type:
            raise("Currently we support only AWS cloud for this SDK.")

        if not name:
            name = f"Ops project {str(round(time.time() * 10000000))}"

        with open(model_file, "rb") as f:
            file_content = f.read()
            return Opsproject(req.post(f"{self.url}/deploy-model-file/",
                                params={"token": self.user_token},
                                files={'file': file_content},
                                data={'token': self.user_token,
                                      'filename': model_file.split("/")[-1] if "/" in model_file else model_file,
                                      'projectName': name,
                                      'serverType': server_type,
                                      'region': region,
                                      }, stream=True).json(), self.user, url=self.url)

    def get_magic_code(self, training_method, data_file, value_for_predict):

        dataconnector = self.create_dataconnector(data_file, has_label_data=True, predict_column_name=value_for_predict)
        print("The data is being processed now. It will take a while. (Mostly less than 5 minutes.)")
        is_uploaded = False
        for i in range(100):
            time.sleep(5)
            dataconnector = self.get_dataconnector(dataconnector.id)
            if dataconnector.status == 100:
                is_uploaded = True
                break

        if not is_uploaded:
            raise ("The training data is being processed now. Please retry with ds2.get_magic_code() when the data is ready. When it is ready, dataconnector.status will return 100.")

        return dataconnector.get_magic_code(training_method, value_for_predict)




    def rent_custom_training_server(self, cloud_type="AWS", region="us-west-1", server_type="g4dn.xlarge",
                                    name=None):
        if not name:
            name = f"Jupyter project {str(round(time.time() * 10000000))}"

        if "AWS" != cloud_type:
            raise("Currently we support only AWS cloud for this SDK.")

        return Jupyterproject(req.post(f"{self.url}/jupyterprojects/",
                         params={"token": self.user_token},
                         data=json.dumps({
                                          'projectName': name,
                                          'region': region,
                                          'serverType': server_type,
                                          })).json(), self.user, url=self.url)

    def predict(self, data, model_id=None, quick_model_name="", return_type="info"):

        if isinstance(data, Labelfile):
            data = data.download()

        if not model_id and not quick_model_name:
            raise("You need to choose model id or quick model name.")

        if model_id:
            model = self.get_model(model_id)

            if model.status != 100:
                raise("This model is not ready yet.")
        else:
            model = self.get_quick_model_by_slug_name(quick_model_name)

        return model.predict(data, return_type=return_type)

    def get_server_lists(self):
        return req.get(f"{self.url}/server-pricing/", params={"token": self.user_token}).json()




import json
from .util import Util
import requests as req


class Dataconnector(object):
    def __init__(self, info, user, url):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.name = info['dataconnectorName']
        self.url = url if url else Util().url
        self.user = user
        self.status = info['status']
        self.user_token = self.user.token

    def __repr__(self):
        return f"{str(self.id)}: {str(self.name)}"

    def delete(self):
        req.delete(f"{self.url}/dataconnectors/{self.id}/",params={"token": self.user_token})

    def get_magic_code(self, training_method, value_for_predict, file_path="output.ipynb"):

        if self.status != 100:
            raise ("The training data is being processed now. Please retry with dataconnector.get_magic_code() when the data is ready. When it is ready, dataconnector.status will return 100.")


        response = req.post(f"{self.url}/get-magic-code/",
                         params={"token": self.user_token},
                         data=json.dumps({
                             'dataconnector': self.id,
                             'trainingMethod': training_method,
                             'valueForPredict': value_for_predict,
                         }))

        if file_path:
            with open(file_path, 'w') as output:
                text = response.json()
                if isinstance(text, dict):
                    text = json.dumps(text)
                output.write(text)

        print(response.json())



from .util import Util, Asynctask, Instance
import requests as req
import json


class Labelproject(object):
    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.workapp = info['workapp']
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token
        labelclasses = []
        if info.get("labelclasses"):
            for labelclass_raw in info.get("labelclasses"):
                labelclasses.append(Labelclass(labelclass_raw, user))
        self.labelclasses = labelclasses

    def __repr__(self):
        return str(self.id)

    def delete(self):
        req.delete(f"{self.url}/labelprojects/{self.id}/",params={"token": self.user_token})

    def get_labels(self):
        labels_raw = req.get(f"{self.url}/labels-by-labelproject/{self.id}/", params={"token": self.user_token}).json()
        labels = []
        for label_raw in labels_raw:
            labels.append(Label(label_raw, self.user, url=self.url))

        return labels

    def get_labelclasses(self):
        return self.labelclasses

    def get_labelfiles(self, sorting="created_at", tab="all", count=10, desc=False, searching="", workAssignee=None):
        labelfiles_raw = req.get(f"{self.url}/listobjects/", params={
                                                                    "token": self.user_token,
                                                                    "labelprojectId": self.id,
                                                                    "sorting": sorting,
                                                                    "tab": tab,
                                                                    "count": count,
                                                                    "desc": desc,
                                                                    "searching": searching,
                                                                    "workAssignee": workAssignee,
                                                                     }).json()
        labelfiles = []
        for label_raw in labelfiles_raw.get('file',[]):
            labelfiles.append(Labelfile(label_raw, self.user, self.labelclasses, url=self.url))

        return labelfiles

    def create_labelclass(self, name, color="#000000"):
        for labelclass in self.labelclasses:
            if name == labelclass.name:
                raise("You can not create same label class name.")

        return Labelclass(req.post(f"{self.url}/labelclasses/",
                         params={"token": self.user_token},
                         data=json.dumps({
                                          'name': name,
                                          'labelproject': self.id,
                                          'color': color
                                          })).json(), self.user, url=self.url)

    def create_labelfile(self, data_file):
        return req.post(f"{self.url}/add-object/", files={'files': open(data_file, "rb")}, data={
                        'token': self.user_token,
                        'labelprojectId': self.id,
                        'frame_value': 0
                    }, stream=True).json()

    def create_custom_ai(self, custom_ai_type="box", use_class_info={},
                         valueForPredictColumnId=None, trainingColumnInfo={}):
        """

            `Create Custom Ai`

            :param item
            - **token**: str = user token

            :json item
            - **custom_ai_type**: str = labeling type. Example, "polygon" or "box"
            - **use_class_info**: dict = labelClass to use for Autolabeling
            - **labelproject_id**: int = target label project id

            \f
        """

        return req.post(f"{self.url}/customai/",
                         params={"token": self.user_token},
                         data=json.dumps({
                                          'custom_ai_type': custom_ai_type,
                                          'use_class_info': use_class_info,
                                          'valueForPredictColumnId': valueForPredictColumnId,
                                          'trainingColumnInfo': trainingColumnInfo,
                                          'labelproject_id': self.id,
                                          })).json()
        pass


    def autolabeling(self, amount, ai_type="general", autolabeling_type="box", general_ai_type="person",
                    model_id=None, custom_ai_stage=0, preprocessing_ai_type={}, labeling_class=None ):
        """
            `Create AutoLabeling`

            - **ai_type**: int =  autolabelingAiType (custom or general or inference)
            - **autolabeling_type**: str = autolabeling type (For example, box or polygon)
            - **custom_ai_stage**: int = CustomAi Count
            - **general_ai_type**: str = None or generalAiType (For example person or road or animal or fire)
            - **preprocessing_ai_type**: dict = autolabeling preprocessingType. For example, {"faceblur": true}
            - **autolabeling_amount**: int = Number of images to autolabeling
            - **labeling_class**: List[str] = List of label classes

            \f
        """

        return req.post(f"{self.url}/autolabeling/",
                         params={"token": self.user_token},
                         data=json.dumps({
                                          'autolabeling_amount': amount,
                                          'autolabeling_ai_type': ai_type,
                                          'autolabeling_type': autolabeling_type,
                                          'custom_ai_stage': custom_ai_stage,
                                          'general_ai_type': general_ai_type,
                                          'preprocessing_ai_type': preprocessing_ai_type,
                                          'labeling_class': labeling_class,
                                          'model_id': model_id,
                                          'labelproject_id': self.id,
                                          })).json()
        pass

    def export(self, is_get_image=False):
        if self.workapp in ['object_detection', 'image']:
            return Asynctask(req.post(f"{self.url}/export-coco/{self.id}/",
                            params={"token": self.user_token, 'is_get_image': is_get_image},
                            ).json(), self.user, url=self.url)
        else:
            return Asynctask(req.post(f"{self.url}/export-data/{self.id}/",
                           params={"token": self.user_token},
                           ).json(), self.user, url=self.url)


class Labelfile(object):
    def __init__(self, info, user, label_classes, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.file_name = info['originalFileName']
        self.status = info['status']
        self.file_url = info['s3key']
        self.width = info['width']
        self.height = info['height']
        self.labelproject = info['labelproject']
        self.label_classes = label_classes
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token

    def __repr__(self):
        return str(self.file_name)

    def download(self, file_path=""):

        if not file_path:
            file_path = self.file_name

        response = req.get(self.file_url)
        with open(file_path, 'wb') as output:
            output.write(response.content)

        return file_path

    def set_done(self, workAssignee=None):
        self.status = "done"
        return self.set_status("done", workAssignee=workAssignee)

    def set_status(self, status, workAssignee=None):
        self.status = status
        return req.put(f"{self.url}/sthreefiles/{self.id}/",
                 params={"token": self.user_token},
                 data=json.dumps({
                     'status': status,
                     'workAssignee': workAssignee,
                 })).json()

    def create_label(self, label, class_name=None, color=None, box=[], polygon=[], structuredData=None):

        if isinstance(label, Instance):
            return self.create_labels([label])
        return self.create_labels([{
                     'labeltype': label,
                     'class_name': class_name,
                     'sthreefile': self.id,
                     'color': color,
                     'box': box,
                     'polygon': polygon,
                     'structuredData': structuredData,
                     'labelproject': self.id,
                     'highlighted': False,
                     'locked': True,
                     'status': "done",
                 }])[0]

    def create_labels(self, labels_raw):
        labels = []
        for label_raw in labels_raw:
            if isinstance(label_raw, Instance):
                label_raw = label_raw.__dict__

            label_class_id = None
            label_class_name = label_raw.get("class_name")
            for label_class in self.label_classes:
                if label_class.name == label_class_name:
                    label_class_id = label_class.id

            if not label_class_id:
                raise Exception("You need to create a label class first.")

            processed_points = []
            for point in label_raw.get("polygon", []):
                label_raw['labeltype'] = "polygon"
                processed_points.append([point[0] / self.width, point[1] / self.height])

            if label_raw.get("box", []):
                label_raw['labeltype'] = "box"
                label_raw['x'] = round(label_raw['box'][0] / self.width, 8)
                label_raw['w'] = round((label_raw['box'][2] - label_raw['box'][0]) / self.width, 8)
                label_raw['y'] = round(label_raw['box'][1] / self.height, 8)
                label_raw['h'] = round((label_raw['box'][3] - label_raw['box'][1]) / self.height, 8)

            label_raw.update({
                              'labelproject': self.labelproject,
                              'labelclass': label_class_id,
                              'sthreefile': self.id,
                              'points': processed_points,
                              'highlighted': False,
                              'locked': True,
                              'status': "done"
                              })
            labels.append(label_raw)

        response = req.post(f"{self.url}/labels/",
                 params={"token": self.user_token, "info": True},
                 data=json.dumps(labels)).json()

        if response.get("result", "") == "success":
            self.set_done()

        return response

    def delete(self):
        return req.delete(f"{self.url}/sthreefiles/",
                          params={"token": self.user_token},
                          data={"sthreefilesId": [str(self.id)]})

class Label(object):
    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.labeltype = info['labeltype']
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token

    def __repr__(self):
        return str(f"{self.labeltype}: {self.id}")

    def delete(self):
        req.delete(f"{self.url}/labels/{self.id}/",params={"token": self.user_token})


class Labelclass(object):
    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.labelproject = info['labelproject']
        self.url = url if url else Util().url
        self.user = user
        self.name = info['name']
        self.user_token = self.user.token

    def __repr__(self):
        return str(self.name)

    def modify(self, name=None, color=None):
        return Labelclass(req.put(f"{self.url}/labelclasses/{self.id}/",
                            params={"token": self.user_token},
                            data=json.dumps({
                                'name': name,
                                'color': color,
                            })).json(), self.user, url=self.url)

    def delete(self):
        req.delete(f"{self.url}/labelclasses/{self.id}/",params={"token": self.user_token})



import time

from .DS2dataset import Dataconnector
from .SkyhubAI import Opsproject
from .util import Util
import requests as req
import json

class Project(object):

    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.url = url if url else Util().url
        self.__dict__.update(info)
        self.user = user
        self.user_token = self.user.token
        self.id = info['id']
        self.status = info['status']
        self.dataconnectorsList = info['dataconnectorsList']
        if isinstance(self.dataconnectorsList[0], int):
            main_dataconnector = self.get_dataconnector(self.dataconnectorsList[0])
        else:
            main_dataconnector = Dataconnector(self.dataconnectorsList[0], self.user, url=self.url)
        models = []
        for model in info.get("models", []):
            models.append(Model(model, user, project=self, main_dataconnector=main_dataconnector))
        self.models = models
        self.jupyter_servers = []
        for jupyter_server in info.get('jupyterServers', []):
            self.jupyter_servers.append(Jupyterserver(jupyter_server, user))

    def __repr__(self):
        return str(self.id)

    def get_dataconnector(self, dataconnector_id):
        return Dataconnector(req.get(f"{self.url}/dataconnectors/{dataconnector_id}/",
                                     params={"token": self.user_token}).json(), self.user, url=self.url)

    def refresh(self):
        return Project(req.get(f"{self.url}/projects/{self.id}/",
                                 params={"token": self.user_token}).json(), self.user, url=self.url)

    def delete(self):
        req.delete(f"{self.url}/projects/{self.id}/",params={"token": self.user_token})

    def train(self, training_method, value_for_predict, option="speed"):

        if self.status != 0:
            raise("The training is already started.")

        return Project(req.post(f"{self.url}/train/{self.id}/",
                            params={"token": self.user_token},
                            data=json.dumps({
                                'trainingMethod': training_method,
                                'valueForPredict': value_for_predict,
                                'option': option,
                            })).json(), self.user, url=self.url)

    def stop(self):

        if self.status == 0:
            raise("The training is not started.")

        return Project(req.put(f"{self.url}/projects/{self.id}/",params={"token": self.user_token},
                data=json.dumps({
                                 "status": 0,
                                 "statusText": "stopped",
                                 })).json(), self.user, url=self.url)

    def get_magic_code(self, training_method, value_for_predict, file_path="output.ipynb"):

        response = req.post(f"{self.url}/get-magic-code/",
                            params={"token": self.user_token},
                            data=json.dumps({
                                'project': self.id,
                                'trainingMethod': training_method,
                                'valueForPredict': value_for_predict,
                            }))

        if file_path:
            with open(file_path, 'w') as output:
                text = response.json()
                if isinstance(text, dict):
                    text = json.dumps(text)
                output.write(text)

        print(response.json())

class Model(object):
    utilClass = Util()

    def __init__(self, info, user, project=None, main_dataconnector=None, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.token = info['token']
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token
        if info.get('trainingMethod'):
            self.app_url = f"https://ds2.ai/instant_use.html/?modeltoken={self.token}&modelid={self.id}"
        else:
            self.app_url = None

        if not project:
            self.project = self.get_project(info['project'])
        if not main_dataconnector:
            self.main_dataconnector = Dataconnector(self.project.dataconnectorsList[0], self.user, url=self.url)

    def __repr__(self):
        return str(self.id)

    def get_project(self, project_id):
        return Project(req.get(f"{self.url}/projects/{project_id}/", params={"token": self.user_token}).json(), self.user, url=self.url)

    def get_dataconnector(self, dataconnector_id):
        return Dataconnector(req.get(f"{self.url}/dataconnectors/{dataconnector_id}/",
                                     params={"token": self.user_token}).json(), self.user, url=self.url)

    def predict(self, data, return_type="info"):

        if self.status != 100:
            raise("This model is not ready yet.")

        request_data = {
            "apptoken": self.user.appTokenCode,
            "modelid": self.id,
            "modeltoken": self.token,
            "userId": self.user.id,
        }
        predict_url = {
            "predict": f"{self.url}/predict/{self.user.id}/",
            "predictimagebyurl": f"{self.url}/predictimagebyurl/{self.user.id}/",
            "predictimagebyurlinfo": f"{self.url}/predictimagebyurlinfo/{self.user.id}/",
            "predictall": f"{self.url}/predictall/{self.user.id}/",
            "predictimage": f"{self.url}/predictimage/{self.user.id}/",
            "predictimageinfo": f"{self.url}/predictimageinfo/{self.user.id}/",
        }

        if type(data) == dict:
            data_processed = {}
            for key, item in data.items():
                if f"__{self.main_dataconnector.dataconnectorName}" not in key:
                    data_processed[f"{key}__{self.main_dataconnector.dataconnectorName}"] = item
                else:
                    data_processed[key] = item
            data = data_processed


        return self.utilClass.predict(request_data, predict_url, data, return_type=return_type)


    def delete(self):
        req.delete(f"{self.url}/models/{self.id}/",params={"token": self.user_token})

    def get_app_url(self):
        if self.app_url:
            return self.app_url
        else:
            print("Currently we don't support the app url for the loaded model.")

    def deploy(self, cloud_type="AWS", region="us-west-1", server_type="g4dn.xlarge", name=None):
        if not name:
            name = f"Ops project {str(round(time.time() * 10000000))}"

        if "AWS" != cloud_type:
            raise("Currently we support only AWS cloud for this SDK.")

        return Opsproject(req.post(f"{self.url}/opsprojects/",
                         params={"token": self.user_token},
                         data=json.dumps({
                                          'projectName': name,
                                          'region': region,
                                          'serverType': server_type,
                                          'modelId': self.id,
                                          })).json(), self.user, url=self.url)


class Jupyterproject(object):
    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token
        jupyterservers = []
        if info.get('jupyterServers'):
            for jupyterserver_raw in info['jupyterServers']:
                jupyterservers.append(Jupyterserver(jupyterserver_raw, user))
        self.jupyterservers = jupyterservers

    def __repr__(self):
        return str(self.id)
    serverType: str = None
    jupyterProjectId: int = None
    region: str = None

    def add_server(self, cloud_type="AWS", region="us-west-1", server_type="g4dn.xlarge"):

        if "AWS" != cloud_type:
            raise("Currently we support only AWS cloud for this SDK.")

        return Jupyterserver(req.post(f"{self.url}/jupyterservers/",
                 params={"token": self.user_token},
                 data=json.dumps({
                     'jupyterProjectId': self.id,
                     'serverType': server_type,
                     'region': region,
                 })).json(), self.user, url=self.url)

    def get_server_status(self):
        return req.get(f"{self.url}/jupyter-servers-status/",
                                 params={"token": self.user_token, "jupyterProjectId": self.id}).json()

    def get_jupyterservers(self):
        return self.jupyterservers

    def refresh(self):
        return Jupyterproject(req.get(f"{self.url}/jupyterprojects/{self.id}/",
                                 params={"token": self.user_token}).json(), self.user, url=self.url)

    def delete(self):
        req.delete(f"{self.url}/jupyterprojects/{self.id}/",params={"token": self.user_token})


class Jupyterserver(object):
    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.instanceId = info['instanceId']
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token

    def __repr__(self):
        return str(self.instanceId)

    def stop(self):
        print(f"{self.instanceId} is stopped.")
        req.post(f"{self.url}/jupyterservers/{self.instanceId}/stop/",
                 params={"token": self.user_token})
        return

    def resume(self):
        print(f"{self.instanceId} is resumed.")
        req.post(f"{self.url}/jupyterservers/{self.instanceId}/resume/",
                                 params={"token": self.user_token})
        return

    def delete(self):
        print(f"{self.instanceId} is deleted. It will take a time to turn off the server.")
        req.delete(f"{self.url}/jupyterservers/{self.instanceId}/",params={"token": self.user_token})
        return



from .DS2dataset import Dataconnector
from .util import Util
import requests as req
import json

class Opsproject(object):
    utilClass = Util()

    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.url = url if url else Util().url
        if info.get('model'):
            self.opsModel = Opsmodel(info['model'], user)
            self.token = self.opsModel.token
            if info.get('trainingMethod'):
                self.app_url = f"https://ds2.ai/service_app.html/?modeltoken={self.token}&modelid={self.id}"
            else:
                self.app_url = None
        self.ops_server_groups = []
        if info.get('opsServerGroupsInfo'):
            for ops_server_group in info.get('opsServerGroupsInfo', []):
                self.ops_server_groups.append(Opsservergroup(ops_server_group, user))
        self.user = user
        self.status = info['status']
        self.dataconnectorsList = info['dataconnectorsList']
        self.user_token = self.user.token
        if isinstance(self.dataconnectorsList[0], int):
            self.main_dataconnector = self.get_dataconnector(self.dataconnectorsList[0])
        else:
            self.main_dataconnector = Dataconnector(self.dataconnectorsList[0], self.user, url=self.url)

    def __repr__(self):
        return str(self.id)

    def get_app_url(self):
        if self.app_url:
            return self.app_url
        else:
            print("Currently we don't support the app url for the loaded model.")

    def delete(self):
        return req.delete(f"{self.url}/opsprojects/{self.id}/",params={"token": self.user_token})

    def get_server_status(self):
        return req.get(f"{self.url}/ops-servers-status/",
                                 params={"token": self.user_token, "opsProjectId": self.id}).json()

    def refresh(self):
        return Opsproject(req.get(f"{self.url}/opsprojects/{self.id}/",
                                 params={"token": self.user_token}).json(), self.user, url=self.url)

    def predict(self, data, return_type="info"):

        if self.status != 100:
            raise("This model is not ready yet.")

        request_data = {
            "apptoken": self.user.appTokenCode,
            "modelid": self.model['id'],
            "modeltoken": self.token,
            "userId": self.user.id,
        }
        predict_url = {
            "predict": f"{self.url}/inference/inferenceops{self.id}/",
            "predictimagebyurl": f"{self.url}/inferenceimagebyurl/inferenceops{self.id}/",
            "predictimagebyurlxai": f"{self.url}/inferenceimagebyurlxai/inferenceops{self.id}/",
            "predictimagebyurlinfo": f"{self.url}/inferenceimagebyurlinfo/inferenceops{self.id}/",
            "predictall": f"{self.url}/inferenceall/inferenceops{self.id}/",
            "predictimage": f"{self.url}/inferenceimage/inferenceops{self.id}/",
            "predictimagexai": f"{self.url}/inferenceimagexai/inferenceops{self.id}/",
            "predictimageinfo": f"{self.url}/inferenceimageinfo/inferenceops{self.id}/",
        }

        if type(data) == dict:
            data_processed = {}
            for key, item in data.items():
                if f"__{self.main_dataconnector.dataconnectorName}" not in key:
                    data_processed[f"{key}__{self.main_dataconnector.dataconnectorName}"] = item
                else:
                    data_processed[key] = item
            data = data_processed

        return self.utilClass.predict(request_data, predict_url, data, return_type=return_type)

    def get_dataconnector(self, dataconnector_id):
        return Dataconnector(req.get(f"{self.url}/dataconnectors/{dataconnector_id}/",
                                     params={"token": self.user_token}).json(), self.user, url=self.url)

class Opsmodel(object):
    utilClass = Util()

    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.token = info['token']
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token

    def __repr__(self):
        return str(self.id)

class Opsservergroup(object):
    def __init__(self, info, user, url=None):
        if not isinstance(info, dict):
            raise Exception(str(info))
        if info.get('error'):
            raise Exception(info['message_en'])
        self.__dict__.update(info)
        self.id = info['id']
        self.url = url if url else Util().url
        self.user = user
        self.user_token = self.user.token

    def __repr__(self):
        return str(self.id)

    def edit_autoscaling(self, min_server_size=None, max_server_size=None, start_server_size=None):
        data = {}
        if min_server_size:
            data['minServerSize'] = min_server_size
        if max_server_size:
            data['maxServerSize'] = max_server_size
        if start_server_size:
            data['startServerSize'] = start_server_size

        return req.put(f"{self.url}/opsservergroups/{self.id}/",
                        params={"token": self.user_token},
                        data = json.dumps(data)
                       ).json()

    def stop(self):
        return req.put(f"{self.url}/opsservergroups/{self.id}/",
                       params={"token": self.user_token},
                       data=json.dumps({
                           'minServerSize': 0,
                           'maxServerSize': 0,
                           'startServerSize': 0,
                       })
                       ).json()

    def resume(self):
        return req.put(f"{self.url}/opsservergroups/{self.id}/",
                       params={"token": self.user_token},
                       data=json.dumps({
                           'minServerSize': 1,
                           'maxServerSize': 1,
                           'startServerSize': 1,
                       })
                       ).json()

    def delete(self):
        return req.delete(f"{self.url}/opsservergroups/{self.id}/",params={"token": self.user_token})
